@@ -45,7 +45,9 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
              * @type {{}[]}
              */
             this.relational_operators = [
-                { label: '=', search: '=', class: 'operator', type: 'operator', default: true },
+                { label: '~', search: '~', class: 'operator', type: 'operator', default: true },
+                { label: '!~', search: '!~', class: 'operator', type: 'operator' },
+                { label: '=', search: '=', class: 'operator', type: 'operator' },
                 { label: '!=', search: '!=', class: 'operator', type: 'operator' },
                 { label: '>', search: '>', class: 'operator', type: 'operator' },
                 { label: '<', search: '<', class: 'operator', type: 'operator' },
@@ -105,36 +107,14 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
             return termIndex;
         }
 
-        updateTermData(termData, input) {
-            super.updateTermData(termData, input);
-
-            if (termData.pattern) {
-                input.pattern = termData.pattern;
-                delete termData.pattern;
-
-                if (termData.invalidMsg) {
-                    input.dataset.invalidMsg = termData.invalidMsg;
-                    delete termData.invalidMsg;
-                }
-
-                if (! this.checkValidity(input)) {
-                    this.reportValidity(input);
-                }
-            }
-        }
-
         readFullTerm(input, termIndex = null) {
             let termData = super.readFullTerm(input, termIndex);
             if (termData === false) {
                 return false;
             }
 
-            if (termData.type === 'terms') {
-                termData = JSON.parse(termData.terms);
-            } else {
-                if (! termData.type) {
-                    termData.type = this.termType;
-                }
+            if (! Array.isArray(termData) && ! termData.type) {
+                termData.type = this.termType;
             }
 
             return termData;
@@ -355,14 +335,6 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
             lastTerm.classList.add('last-term');
         }
 
-        saveTerm(input, updateDOM = true) {
-            if (! this.checkValidity(input)) {
-                return false;
-            }
-
-            return super.saveTerm(input, updateDOM);
-        }
-
         termsToQueryString(terms) {
             if (! this.input.form.checkValidity()) {
                 let filtered = [];
@@ -386,8 +358,7 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
             let termIndex = Number(label.dataset.index);
             if (termIndex < this.usedTerms.length - 1) {
                 // It's not the last term
-                if (! this.checkValidity(label.firstChild, label.dataset.type, termIndex)) {
-                    this.reportValidity(label.firstChild);
+                if (! this.validate(label.firstChild)) {
                     return false;
                 }
             }
@@ -546,8 +517,8 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
                 case 'operator':
                 case 'logical_operator':
                     let suggestions = this.validOperator(
-                        data.trigger === 'script' ? '': data.term.label, data.term.type, termIndex);
-                    if (suggestions.exactMatch) {
+                        data.trigger === 'script' ? '' : data.term.label, data.term.type, termIndex);
+                    if (suggestions.exactMatch && ! suggestions.partialMatches) {
                         // User typed a suggestion manually, don't show the same suggestion again
                         return;
                     }
@@ -709,7 +680,7 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
                 operators = filtered;
             }
 
-            operators.exactMatch = exactMatch && ! partialMatch;
+            operators.exactMatch = exactMatch;
             operators.partialMatches = partialMatch;
 
             return operators;
@@ -834,25 +805,17 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
         }
 
         checkValidity(input, type = null, termIndex = null) {
+            if (! super.checkValidity(input)) {
+                return false;
+            }
+
             if (type === null) {
                 type = input.parentNode.dataset.type;
             }
 
-            if (input.pattern && ! input.checkValidity()) {
-                if (! input.value.match(input.pattern)) {
-                    if (input.dataset.invalidMsg) {
-                        input.setCustomValidity(input.dataset.invalidMsg);
-                    }
-
-                    return false;
-                }
-
-                input.setCustomValidity('');
-            }
-
             if (! type || type === 'value') {
                 // type is undefined for the main input, values have no special validity rules
-                return input.checkValidity();
+                return true;
             }
 
             if (termIndex === null && input.parentNode.dataset.index >= 0) {
@@ -878,7 +841,7 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
                 }
             } else {
                 let isRequired = ! options.exactMatch;
-                if ((options.partialMatches && options.length > 1) || (type === 'negation_operator' && ! value)) {
+                if (type === 'negation_operator' && ! value) {
                     isRequired = false;
                 } else if (type === 'operator' && ! value) {
                     let nextTermAt = termIndex + 1;
@@ -949,10 +912,6 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
             return input.checkValidity();
         }
 
-        reportValidity(element) {
-            setTimeout(() => element.reportValidity(), 0);
-        }
-
         renderSuggestions(suggestions) {
             let itemTemplate = $.render('<li><input type="button" tabindex="-1"></li>');
 
@@ -1009,11 +968,16 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
             return label;
         }
 
-        autoSubmit(input, changeType, changedTerms) {
+        autoSubmit(input, changeType, data) {
             if (this.shouldNotAutoSubmit()) {
                 return;
             }
 
+            let changedTerms = [];
+            if ('terms' in data) {
+                changedTerms = data['terms'];
+            }
+
             let changedIndices = Object.keys(changedTerms).sort((a, b) => a - b);
             if (! changedIndices.length) {
                 return;
@@ -1026,7 +990,7 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
                     lastTermAt = changedIndices.pop();
                     if (changedTerms[lastTermAt].type === 'value') {
                         if (! changedIndices.length) {
-                            changedTerms = {
+                            data['terms'] = {
                                 ...{
                                     [lastTermAt - 2]: this.usedTerms[lastTermAt - 2],
                                     [lastTermAt - 1]: this.usedTerms[lastTermAt - 1]
@@ -1055,35 +1019,35 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
                     return;
                 case 'save':
                     let updateAt = changedIndices[0];
-                    if (typeof this.usedTerms[updateAt] !== 'undefined'
-                        && this.usedTerms[updateAt] === changedTerms[updateAt]
-                    ) {
-                        let valueAt = updateAt;
-                        switch (changedTerms[updateAt].type) {
-                            case 'column':
+                    let valueAt = updateAt;
+                    switch (changedTerms[updateAt].type) {
+                        case 'column':
+                            if (changedTerms[updateAt].label !== this.usedTerms[updateAt].label) {
                                 return;
-                            case 'operator':
-                                valueAt++;
-                        }
-
-                        if (valueAt === updateAt) {
-                            if (changedIndices.length === 1) {
-                                changedTerms = {
-                                    ...{
-                                        [valueAt - 2]: this.usedTerms[valueAt - 2],
-                                        [valueAt - 1]: this.usedTerms[valueAt - 1]
-                                    },
-                                    ...changedTerms
-                                };
                             }
 
-                            break;
-                        } else if (this.usedTerms.length > valueAt && this.usedTerms[valueAt].type === 'value') {
-                            break;
+                            valueAt++;
+                        case 'operator':
+                            valueAt++;
+                    }
+
+                    if (valueAt === updateAt) {
+                        if (changedIndices.length === 1) {
+                            data['terms'] = {
+                                ...{
+                                    [valueAt - 2]: this.usedTerms[valueAt - 2],
+                                    [valueAt - 1]: this.usedTerms[valueAt - 1]
+                                },
+                                ...changedTerms
+                            };
                         }
 
-                        return;
+                        break;
+                    } else if (this.usedTerms.length > valueAt && this.usedTerms[valueAt].type === 'value') {
+                        break;
                     }
+
+                    return;
                 case 'remove':
                     let firstTermAt = changedIndices.shift();
                     if (changedTerms[firstTermAt].type === 'column'
@@ -1097,7 +1061,7 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
                     return;
             }
 
-            super.autoSubmit(input, changeType, changedTerms);
+            super.autoSubmit(input, changeType, data);
         }
 
         encodeTerm(termData) {
@@ -1114,6 +1078,10 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
             return termData;
         }
 
+        isTermDirectionVertical() {
+            return false;
+        }
+
         highlightTerm(label, highlightedBy = null) {
             label.classList.add('highlighted');
 
@@ -1247,22 +1215,19 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
 
         onTermFocus(event) {
             let input = event.target;
+            let isTerm = input.parentNode.dataset.index >= 0;
             let termType = input.parentNode.dataset.type || this.termType;
 
-            if (input.parentNode.dataset.index >= 0) {
-                if (
-                    ! this.checkValidity(input, termType)
-                    && termType !== 'operator'
-                    && termType !== 'logical_operator'
-                ) {
-                    this.reportValidity(input);
-                }
-
+            if (isTerm) {
                 this.highlightTerm(input.parentNode);
             }
 
             let value = this.readPartialTerm(input);
             if (! value && (termType === 'column' || termType === 'value')) {
+                if (isTerm) {
+                    this.validate(input);
+                }
+
                 // No automatic suggestions without input
                 return;
             }
@@ -1349,17 +1314,14 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
                 }
             }
 
-            this.autoSubmit(this.input, 'remove', this.removeRange(labels));
+            this.autoSubmit(this.input, 'remove', { terms: this.removeRange(labels) });
             this.togglePlaceholder();
         }
 
         onCompletion(event) {
             super.onCompletion(event);
 
-            let input = event.target;
-            this.checkValidity(input);
-
-            if (input.parentNode.dataset.index >= 0) {
+            if (event.target.parentNode.dataset.index >= 0) {
                 return;
             }
 
@@ -1445,9 +1407,10 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
 
             if (isTerm) {
                 let newTerm = null;
-                if (operators.exactMatch && operators[0].label.toLowerCase() !== value.toLowerCase()) {
+                let exactMatchOnly = operators.exactMatch && ! operators.partialMatches;
+                if (exactMatchOnly && operators[0].label.toLowerCase() !== value.toLowerCase()) {
                     // The user completes a partial match
-                } else if (operators.exactMatch && (termType !== 'operator' || operators[0].type !== 'operator')) {
+                } else if (exactMatchOnly && (termType !== 'operator' || operators[0].type !== 'operator')) {
                     newTerm = { ...operators[0] };
                 } else if (operators.partialMatches && termType !== 'operator') {
                     newTerm = { ...operators[0], label: value, search: value };
@@ -1466,7 +1429,7 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
 
                 if (newTerm !== null) {
                     let label = this.insertTerm(newTerm, termIndex + 1);
-                    this.autoSubmit(label.firstChild, 'insert', { [termIndex + 1]: newTerm });
+                    this.autoSubmit(label.firstChild, 'insert', { terms: { [termIndex + 1]: newTerm } });
                     this.complete(label.firstChild, { term: newTerm });
                     $(label.firstChild).focus({ scripted: true });
                     event.preventDefault();
@@ -1477,13 +1440,13 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
                     this.togglePlaceholder();
                 } else if (operators.exactMatch) {
                     if (termType !== operators[0].type) {
-                        this.autoSubmit(input, 'exchange', this.exchangeTerm());
+                        this.autoSubmit(input, 'exchange', { terms: this.exchangeTerm() });
                     } else {
                         this.clearPartialTerm(input);
                     }
 
                     this.addTerm({ ...operators[0] });
-                    this.autoSubmit(input, 'add', { [this.usedTerms.length - 1]: operators[0] });
+                    this.autoSubmit(input, 'add', { terms: { [this.usedTerms.length - 1]: operators[0] } });
                     this.togglePlaceholder();
                     event.preventDefault();
                 } else if (termType === 'operator') {
@@ -1499,21 +1462,14 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
 
         onInput(event) {
             let input = event.target;
-            let termIndex = Number(input.parentNode.dataset.index);
-
             if (this.hasSyntaxError(input)) {
-                // pass
-            } else if (termIndex >= 0) {
-                let value = this.readPartialTerm(input);
-                if (! this.checkValidity(input)) {
-                    this.reportValidity(input);
-                    // Let inputs also grow upon invalid input
-                    this.updateTermData({ label: value }, input);
-                    return;
-                } else if (value && ! ['column', 'value'].includes(input.parentNode.dataset.type)) {
-                    this.autoSubmit(input, 'save', { [termIndex]: this.saveTerm(input) });
-                }
-            } else if (this.termType === 'operator' || this.termType === 'logical_operator') {
+                return super.onInput(event);
+            }
+
+            let termIndex = Number(input.parentNode.dataset.index);
+            let isTerm = termIndex >= 0;
+
+            if (! isTerm && (this.termType === 'operator' || this.termType === 'logical_operator')) {
                 let value = this.readPartialTerm(input);
 
                 if (value && ! this.validOperator(value).partialMatches) {
@@ -1532,11 +1488,19 @@ define(["../notjQuery", "BaseInput"], function ($, BaseInput) {
             }
 
             super.onInput(event);
+
+            if (isTerm && input.checkValidity()) {
+                let value = this.readPartialTerm(input);
+                if (value && ! ['column', 'value'].includes(input.parentNode.dataset.type)) {
+                    this.autoSubmit(input, 'save', { terms: { [termIndex]: this.saveTerm(input) } });
+                }
+            }
         }
 
         onPaste(event) {
             if (! this.hasTerms()) {
-                super.onPaste(event);
+                this.submitTerms(event.clipboardData.getData('text/plain'));
+                event.preventDefault();
             } else if (! this.input.value) {
                 let terms = event.clipboardData.getData('text/plain');
                 if (this.termType === 'logical_operator') {
